package main

import (
	"archive/tar"
	"compress/gzip"
	"io"
	"log"
	"net/http"
	"os"
	"strings"
)

func LogMiddleware(next http.Handler) http.Handler {
	return http.HandlerFunc(
		func(w http.ResponseWriter, r *http.Request) {
			defer func() {
				r := recover()
				if r != nil {
					log.Println("panic", r)
					w.WriteHeader(500)
				}
			}()
			next.ServeHTTP(w, r)
			log.Printf("%s %s %s %s\n", r.RemoteAddr, r.Method, r.URL, r.Header.Get("User-Agent"))
		},
	)
}

func bundle(buf io.Writer, paths []string, flat bool) error {
	gw := gzip.NewWriter(buf)
	defer gw.Close()
	tw := tar.NewWriter(gw)
	defer tw.Close()

	for _, file := range paths {
		if err := func(file string) error {
			f, err := os.Open(file)
			if err != nil {
				return err
			}
			defer f.Close()

			info, err := f.Stat()
			if err != nil {
				return err
			}

			header, err := tar.FileInfoHeader(info, info.Name())
			if err != nil {
				return err
			}

			// Use full path as name (FileInfoHeader only takes the basename)
			// If we don't do this the directory strucuture would
			// not be preserved
			// https://golang.org/src/archive/tar/common.go?#L626
			if !flat {
				if !strings.HasPrefix(file, "/") {
					file = "configui/" + file
				} else {
					file = "configui" + file
				}
				header.Name = file
			}

			// Write file header to the tar archive
			err = tw.WriteHeader(header)
			if err != nil {
				return err
			}

			// Copy file content to tar archive
			_, err = io.Copy(tw, f)
			if err != nil {
				return err
			}
			return nil

		}(file); err != nil {
			return err
		}
	}

	return nil
}

func HttpWriter(w http.ResponseWriter, status int, errStr string) {
	w.WriteHeader(status)
	w.Write([]byte(errStr))
}
